@@ -60,9 +60,9 @@ true
 isone(x) = x == one(x) # fallback method
 
 size(x::Number) = ()
-size(x::Number,d) = convert(Int,d)<1 ? throw(BoundsError()) : 1
+size(x::Number, d::Integer) = d < 1 ? throw(BoundsError()) : 1
 axes(x::Number) = ()
-axes(x::Number,d) = convert(Int,d)<1 ? throw(BoundsError()) : OneTo(1)
+axes(x::Number, d::Integer) = d < 1 ? throw(BoundsError()) : OneTo(1)
 eltype(::Type{T}) where {T<:Number} = T
 ndims(x::Number) = 0
 ndims(::Type{<:Number}) = 0
@@ -80,7 +80,7 @@ function getindex(x::Number, i::Integer)
 end
 function getindex(x::Number, I::Integer...)
     @_inline_meta
-    @boundscheck all([i == 1 for i in I]) || throw(BoundsError())
+    @boundscheck all(isone, I) || throw(BoundsError())
     x
 end
 first(x::Number) = x
@@ -216,6 +216,9 @@ julia> inv(1 + 2im) * (1 + 2im)
 julia> inv(2//3)
 3//2
 ```
+
+!!! compat "Julia 1.2"
+    `inv(::Missing)` requires at least Julia 1.2.
 """
 inv(x::Number) = one(x)/x
 
@@ -228,7 +231,7 @@ Multiply `x` and `y`, giving the result as a larger type.
 # Examples
 ```jldoctest
 julia> widemul(Float32(3.), 4.)
-1.2e+01
+12.0
 ```
 """
 widemul(x::Number, y::Number) = widen(x)*widen(y)
@@ -320,8 +323,6 @@ julia> import Dates; oneunit(Dates.Day)
 oneunit(x::T) where {T} = T(one(x))
 oneunit(::Type{T}) where {T} = T(one(T))
 
-_default_type(::Type{Number}) = Int
-
 """
     big(T::Type)
 
